@@ -8,8 +8,8 @@ import warnings
 import numpy as np
 from scipy.linalg.blas import get_blas_funcs
 
-__all__ = ['upsample_stim', 'downsample_stim', 'slicestim', 'getcov',
-           'rolling_window']
+__all__ = ['upsample_stim', 'downsample_stim', 'slicestim', 
+            'cov', 'rolling_window']
 
 
 def upsample_stim(stim, upsample_factor, time=None):
@@ -133,19 +133,21 @@ def slicestim(stimulus, history):
     """
 
     if not (1 <= history <= stimulus.shape[0]):
-        raise ValueError(
-                '`history` must be between 1 and {0:#d}'.format(
-                stimulus.shape[0]))
-    history = int(history)
+        msg = '`history` must be between 1 and {0:#d}'.format(stimulus.shape[0])
+        raise ValueError(msg)
+    elif not isinstance(history, int):
+        raise ValueError("`history` must be an integer")
 
     # Use strides to create view onto array
-    shape = (history, stimulus.shape[0] - history) + stimulus.shape[1:]
-    strides = (stimulus.strides[0],) + stimulus.strides
-    return np.lib.stride_tricks.as_strided(stimulus, 
-            shape=shape, strides=strides)
+    shape = (history, stimulus.shape[0] - history + 1) + stimulus.shape[1:]
+    stride = (stimulus.strides[0],) + stimulus.strides
+
+    # return the newly strided array
+    arr = np.lib.stride_tricks.as_strided(stimulus, shape=shape, strides=stride)
+    return np.rollaxis(arr, 1)
 
 
-def getcov(stimulus, history, tproj=None, verbose=False):
+def cov(stimulus, history, nsamples=None, verbose=False):
     """
     Computes a stimulus covariance matrix
 
@@ -160,62 +162,13 @@ def getcov(stimulus, history, tproj=None, verbose=False):
     history : int
         Integer number of time points to keep in each slice.
 
-    tproj : array_like, optional
-        Temporal basis set to use. Size of the second dimension must be equal
-        to `history`. Each extracted stimulus slice is projected onto this 
-        basis set, which reduces the size of the corresponding covariance 
-        matrix to store.
-
-    verbose : boolean, optional
-        If True, print out progress of the computation. (defaults to False)
-
     Returns
     ------
     stim_cov : array_like
-        (n*n*t by n*n*t) Covariance matrix
-
+        Covariance matrix
     """
-    if not (1 <= history < stimulus.shape[0]):
-        raise ValueError('`history` must be in [1, {0:#d})'.format(
-                stimulus.shape[0]))
-
-    # Collapse spatial dimensions
-    cstim = stimulus.reshape(stimulus.shape[0], -1)
-
-    # store mean + covariance matrix
-    mean = np.zeros(cstim.shape[-1] * history)
-    stim_cov = np.zeros((cstim.shape[-1] * history,) * 2)
-
-    # Update covariance matrix from random points in the stimulus
-    indices = np.arange(history, cstim.shape[0])
-    numpts = np.min((cstim.shape[1] * 10, indices.size))
-    np.random.shuffle(indices)
-
-    # BLAS rank-1 covariance update function
-    blas_ger_fnc = get_blas_funcs(('ger',), (stim_cov,))[0]
-
-    for j in range(numpts):
-        idx = indices[j]
-        if verbose and np.mod(j, 100) == 0:
-            print('[%i of %i]' % (j, numpts))
-
-        stimslice = np.reshape(cstim[idx - history : idx, :], 
-                (-1, 1), order='F')
-
-        mean += np.squeeze(stimslice)
-
-        # Update covariance matrix
-        blas_ger_fnc(1, stimslice, stimslice, 
-                a=stim_cov.T, overwrite_a=True)
-
-    # Normalize and compute the mean outer product
-    mean /= numpts
-    mean_op = mean.reshape(-1, 1).dot(mean.reshape(1, -1))
-
-    # Return normalized covariance matrix
-    stim_cov = (stim_cov / (numpts - 1)) - mean_op
-
-    return stim_cov
+    stim = slicestim(stimulus, history)
+    return np.cov(stim.reshape(stim.shape[0], -1).T)
 
 
 def rolling_window(array, window, time_axis=0):
